`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineers: Mircea Dabacan
//				  Mihai Corovei
//				  Josh Sackos
//
// 
// Create Date:    15:32:13 06/05/2012
// Module Name:    HB5RefComp - Behavioral 
// Project Name:   Hb5RefProj
// Target Devices: Nexys3
// Tool versions:  ISE 14.1
// Description: This is the simple Digilent HB5 Reference Component.
//              This component can also be used with PmodHB3.
//   Inputs:
//     - ck = clock signal (100MHz)
//     - bitDirIn = the direction to rotate the motor 
//     - vecDfIn = the value of the duty factor for the PWM signal.
//   Outputs:
//     - bitEnOut = Enable pin for Pmod HB5
//     - bitDirOut = Dir pin for Pmod HB5
//
//   Internal constants:
//     - intPreDiv = Pre division factor. Defines the division factor
//       used to build the internal clock (ckInt).
//       frequency(ckInt) =  frequency(ckInt)/intPreDiv.
//     - strRadix = string to define the radix used for vecDfIn and 
//       internal counter. Allowed values:
//       - "Bcd" for BCD (the whole PWM circuit works in BCD)
//       - "Hex" for hexadecimal (the whole PWM circuit works in hexadecimal)
//     - intSwDelay = number of ck periods to force bitEnOut "low" before and after
//       a state change of bitDirOut.
//     Behavioral description:
//       bitEnOut is built as a PWM signal. 
//       The frequency of bitEnOut is:
//       - frequency(ckInt)/256 - if strRadix is "Hex" 
//       - frequency(ckInt)/100 - if strRadix is "Bcd" 
//       The Duty Factor of bitEnOut is:
//       - vecDfIn/256 - if strRadix is "Hex" 
//       - vecDfIn/100 - if strRadix is "Bcd" 
//       
//       To avoid Pmod HB5 damage, bitDirOut is not allowed to change state when 
//       bitEnOut is "high" or even close to that. 
//       To make sure of this condition, when the bitDirIn change state, 
//       bitEnOut is forced "low" for intSwDelay ck periods, then bitDirOut gets
//       the value of bitDirIn, bitEnOut is forced "low" for other intSwDelay ck 
//       periods before being released to generate further PWM cycles.                          

// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

// Module name and port list
module HB5RefComp(
    ck,
    bitDirIn,
    vecDfIn,
    bitDirOut,
    bitEnOut
    );

	 // ===========================================================================
	 // 										Port Declarations
	 // ===========================================================================
    input ck;
    input bitDirIn;
    input [7:0] vecDfIn;
    output bitDirOut;
    output bitEnOut;
	 reg bitDirOut;
	 wire bitEnOut;
	 
	 // ===========================================================================
	 // 							  Parameters, Regsiters, and Wires
	 // ===========================================================================
	 parameter intPreDiv = 'd2048;  	  // Predivision factor
												  // with intPreDiv = 2048:
												  // frequency(ckInt) = 100MHz/2048 = 48.83KHz
												  // frequency(bitEnOut) = 100MHz/2048/100 = 488.3Hz (with strRadix = BCD)
												  // frequency(bitEnOut) = 100MHz/2048/256 = 190.7Hz (with strRadix = HEX)

	 parameter [8*3:1] strRadix = "BCD";		  // used radix ("BCD" or "HEX")

    parameter intSwDelay = 'd100000; 	  // # of ck periods to force bitEnOut "low"
													  // before and after a change of bitDirOut state.

	 reg [10:0] cntPreDiv;
	 reg ckInt;
	 wire ckIntBuf;
	 reg [17:0] cntSwDelay = 18'd0;
	 reg [7:0] cntPwm;
	 wire bitPwmInt;
	 reg bitDirOld;

	 // ===========================================================================
	 // 										Implementation
	 // ===========================================================================

	 // BUFG: Global Clock Buffer
    //       Spartan-6
    // Xilinx HDL Language Template, version 13.4

    BUFG BUFG_inst (
       .O(ckIntBuf), // 1-bit output: Clock buffer output
       .I(ckInt)  // 1-bit input: Clock buffer input
    );

    // End of BUFG_inst instantiation

	 // preDiv
	 always @(posedge ck)
	 begin
			if (cntPreDiv == (intPreDiv - 1'b1))
				begin
					cntPreDiv <= 'd0;
					ckInt <= 1'd1;
				end
			else
				begin
					cntPreDiv <= cntPreDiv + 1'b1;
					ckInt <= 1'd0;
				end
	 end

	 // PWM
	 always @(posedge ckIntBuf)
	 begin
			if(strRadix == "HEX")
				begin
					cntPwm <= cntPwm + 4'h1;
				end

			else if(strRadix == "BCD")
				begin

					if(cntPwm[3:0] == 4'h9)
						begin
							cntPwm[3:0] <= 4'h0;
							
							if(cntPwm[7:4] == 4'h9)
							begin
								cntPwm[7:4] <= 4'h0;
							end

							else
							begin
								cntPwm[7:4] <= cntPwm[7:4] + 4'h1;
							end
						end

					else
						begin
							cntPwm[3:0] <= cntPwm[3:0] + 4'h1;
						end
				end

			else
				begin
					cntPwm <= 8'hFF;
				end
	 end

	 // Internal PWM signal
	 assign bitPwmInt = (cntPwm < vecDfIn) ? 1'b1 : 1'b0;

	 // trackDir
	 always @(posedge ck)
	 begin
			bitDirOld <= bitDirIn;
			
			if(bitDirOld != bitDirIn)
				begin
						cntSwDelay <= 4'h0;
				end
			else if(cntSwDelay == 2*intSwDelay)
				begin
						// Null statement here
				end
			else
				begin
						cntSwDelay <= cntSwDelay + 1'b1;
				end
	 end
	 
	 //  Internal output enable
	 assign bitEnOut = (cntSwDelay == 2*intSwDelay) ? bitPwmInt : 1'b0;  //  Normal Operation : During Switching Delay

	 // switchDirection
	 always @(posedge ck)
	 begin
			if(cntSwDelay == intSwDelay)
				begin
						bitDirOut <= bitDirOld;
				end
	 end

endmodule
