`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineers: Mircea Dabacan
//				  Mihai Corovei
//				  Josh Sackos
//
// 
// Create Date:    11:50:35 06/06/2012 
// Module Name:    HallTradSpeedDecoder - Behavioral 
// Project Name:   Hb5RefProj
// Target Devices: Nexys3
// Tool versions:  ISE 14.1
// Description: 
// 	Samples inputs SA and SB from the Hall transducers, on ck rising edge.
//    It divides System Clock in order to obtain a TimeBase for Speed Counter.
//    Based on the SA and SB sequence, increments/decrements the cntSpeed counter.
//    When TimeBase counter finished counting, cntSpeed value is saved in regSpeed
//    and cntSpeed is reset.
//    Measuring TimeBase: a 360 degree motor rotation = (Reduction Factor * 12) cntSpeed counts
//                    (6 edges detected on each SA and SB transducer signal) 
//                        TimeBase = (60 / (Reduction Factor * 12)) sec
//                               => DivFactor =  26315789 for Reduction Factor = 19.
//    Measuring Direction:  On each edge on pinSA or pinSB signal the other one is 
//       verified and according to its value cntSpeed is incremented or decremented. 
//
//    cntSpeed = 0000 if motor off
//               0000 to 0999 if motor rotates forwards
//               1999 to 1000 if motor rotates backwards
//                  (represented in BCD Two's Complement Representation Code)            
// 
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module HallTradSpeedDecoder(
    ck,
    pinSA,
    pinSB,
    SpeedOut
    );

	 // ===========================================================================
	 // 										Port Declarations
	 // ===========================================================================
    input ck;
    input pinSA;
    input pinSB;
    output [12:0] SpeedOut;

	 // ===========================================================================
	 // 							  Parameters, Regsiters, and Wires
	 // ===========================================================================
	 wire [12:0] SpeedOut;
	 
	 reg regSA = 1'b0, regSB = 1'b0;					// Registered values of transducer signals
	 reg oldSA = 1'b0, oldSB = 1'b0;					// Old values of transducer signals
	 reg oldSA2 = 1'b0, oldSB2 = 1'b0;				// Second old values of transducer signals
	 reg [12:0] cntSpeed = 13'b0000000000000;		// Speed counter
	 reg [12:0] regSpeed = 13'b0000000000000;		// Speed register
	 wire cntSpeedEn;										// Save and reset cntSpeed signal

	 parameter DivFactor = 'd26315789;				// Division factor
	 reg [24:0] TimeBaseCounter  = 1'd0;			// TimeBase Counter

	 // ===========================================================================
	 // 										Implementation
	 // ===========================================================================

		//-------------------
		// Signal assignation
		//-------------------

		// cntSpeed Enable signal
		assign cntSpeedEn = (TimeBaseCounter == 1'd0) ? 1'b1 : 1'b0;

		// Convert Speed from BCD Two's Complement to Magnitude and Sign Representation Code.
		assign SpeedOut[11:0] = (regSpeed[12] == 1'b0) ? regSpeed[11:0] : (12'h999 - regSpeed[11:0]);

		assign SpeedOut[12] = regSpeed[12];


		//-----------------------------
		// System Clock Devider Process
		// 	TimeBaseCounterProc
		//-----------------------------
	   always @(posedge ck)
		begin
				if(TimeBaseCounter != DivFactor - 1'b1)
					begin
							TimeBaseCounter <= TimeBaseCounter + 1'b1;
					end
				else
					begin
							TimeBaseCounter <= 1'b0;
					end
		end
		
		//------------------------
		// Measuring Speed Process
		//		SpeedCounter
		//------------------------	
	   always @(posedge ck)
		begin
			
				regSA <= pinSA;  // registering the inputs
				regSB <= pinSB;
				oldSA <= regSA;  // storing values for next iteration
				oldSB <= regSB;
				oldSA2 <= oldSA; // storing values for next iteration second back-up
				oldSB2 <= oldSB;

				if(cntSpeedEn == 1'b1)
					begin
						cntSpeed <= 13'd0;
						regSpeed <= cntSpeed;
					end
				
				else 	
					begin
						if(oldSA2 == 1'b0 && oldSA == 1'b1 && regSA == 1'b1) 				// SA rising edge
							begin
								if(regSB == 1'b0)														// SB = 0 
									begin
										if(cntSpeed[3:0] != 4'h9)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] + 4'h1;			// increment speed counter
											end
										else if(cntSpeed[7:4] != 4'h9)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] + 4'h1;
												cntSpeed[3:0] <= 4'h0;
											end
										else if(cntSpeed[11:8] != 4'h9)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] + 4'h1;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
										else 
											begin
												cntSpeed[12] <= 1'b0;
												cntSpeed[11:8] <= 4'h0;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
									end
								else
									begin
									
										if(cntSpeed[3:0] != 4'h0)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] - 4'h1; // decrement speed counter
											end
											
										else if(cntSpeed[7:4] != 4'h0)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] - 4'h1;
												cntSpeed[3:0] <= 4'h9;
											end
											
										else if(cntSpeed[11:8] != 4'h0)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] - 4'h1;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end
											
										else	
											begin
												cntSpeed[12] <= 1'b1;
												cntSpeed[11:8] <= 4'h9;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end		// Decrement speed counter
									end
							end		


						else if(oldSA2 == 1'b1 && oldSA == 1'b0 && regSA == 1'b0) 		// SA falling edge
							begin
							
								if(regSB == 1'b0)														// SB = 0 
									begin
									
										if(cntSpeed[3:0] != 4'h0)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] - 4'h1; 		// decrement speed counter
											end
											
										else if(cntSpeed[7:4] != 4'h0)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] - 4'h1;
												cntSpeed[3:0] <= 4'h9;
											end
											
										else if(cntSpeed[11:8] != 4'h0)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] - 4'h1;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end
											
										else
											begin
												cntSpeed[12] <= 1'b1;
												cntSpeed[11:8] <= 4'h9;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end		// decrement speed counter
									end
									
								else
									begin
									
										if(cntSpeed[3:0] != 4'h9)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] + 4'h1;		// increment speed counter
											end
											
										else if(cntSpeed[7:4] != 4'h9) 
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] + 4'h1;
												cntSpeed[3:0] <= 4'h0;
											end
											
										else if(cntSpeed[11:8] != 4'h9)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] + 4'h1;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
											
										else 
											begin
												cntSpeed[12] <= 1'b1;
												cntSpeed[11:8] <= 4'h0;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
									end
							end



						if(oldSB2 == 1'b0 && oldSB == 1'b1 && regSB == 1'b1) 	// SB rising edge
							begin
								if(regSA == 1'b1)											// SA = 1 
									begin
										if(cntSpeed[3:0] != 4'h9)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] + 4'h1;	// increment speed counter
											end
										else if(cntSpeed[7:4] != 4'h9)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] + 4'h1;
												cntSpeed[3:0] <= 4'h0;
											end
										else if(cntSpeed[11:8] != 4'h9)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] + 4'h1;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
										else 
											begin
												cntSpeed[12] <= 4'h0;
												cntSpeed[11:8] <= 4'h0;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
									end
								else
									begin
										if(cntSpeed[3:0] != 4'h0)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] - 4'h1; // decrement speed counter
											end
										else if(cntSpeed[7:4] != 4'h0) 
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] - 4'h1;
												cntSpeed[3:0] <= 4'h9;
											end
										else if(cntSpeed[11:8] != 4'h0)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] - 4'h1;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end
										else	
											begin
												cntSpeed[12] <= 1'b1;
												cntSpeed[11:8] <= 4'h9;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end		// decrement speed counter
									end
							end
						else if(oldSB2 == 1'b1 && oldSB == 1'b0 && regSB == 1'b0) 	// SB falling edge
							begin
								if(regSA == 1'b1)													// SA = 1 
									begin
										if(cntSpeed[3:0] != 4'h0)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] - 4'h1; 	// decrement speed counter
											end
										else if(cntSpeed[7:4] != 4'h0)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] - 4'h1;
												cntSpeed[3:0] <= 4'h9;
											end
										else if(cntSpeed[11:8] != 4'h0)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] - 4'h1;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end
										else	
											begin
												cntSpeed[12] <= 4'h1;
												cntSpeed[11:8] <= 4'h9;
												cntSpeed[7:4] <= 4'h9;
												cntSpeed[3:0] <= 4'h9;
											end				// decrement speed counter
									end
								else
									begin
										if(cntSpeed[3:0] != 4'h9)
											begin
												cntSpeed[3:0] <= cntSpeed[3:0] + 4'h1;		// increment speed counter
											end
										else if(cntSpeed[7:4] != 4'h9)
											begin
												cntSpeed[7:4] <= cntSpeed[7:4] + 4'h1;
												cntSpeed[3:0] <= 4'h0;
											end
										else if(cntSpeed[11:8] != 4'h9)
											begin
												cntSpeed[11:8] <= cntSpeed[11:8] + 4'h1;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
										else 
											begin
												cntSpeed[12] <= 1'b0;
												cntSpeed[11:8] <= 4'h0;
												cntSpeed[7:4] <= 4'h0;
												cntSpeed[3:0] <= 4'h0;
											end
									end
							end
				end
		end

endmodule
