`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineers: Mircea Dabacan
//				  Mihai Corovei
//				  Josh Sackos
//
// 
// Create Date:    10:16:38 06/05/2012
// Module Name:    segDisplay
// Project Name:   Hb5RefProj
// Target Devices: Nexys3
// Tool versions:  ISE 14.1
// Description: This module decodes and outputs on the SSG Display the direction and the
//              Speed (rot/min) or the Duty Factor selected using the switches on-board.
//
// Dependencies:
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

// Module name and port list
module segDisplay(
	ck,
	swDir,
	swSelDisp,
	vecDfIn,
	SpeedIn,
	an,
	seg
);

	 // ===========================================================================
	 // 										Port Declarations
	 // ===========================================================================
    input ck;
    input swDir;
    input swSelDisp;
    input [7:0] vecDfIn;
    input [12:0] SpeedIn;
    output [3:0] an;
    output [6:0] seg;

	 // ===========================================================================
	 // 									  Wires and Registers
	 // ===========================================================================	 
	 reg [3:0] an;
	 wire [6:0] seg;
	 reg [6:0] seg1;				// Segments for DF, position, etc.
	 reg [6:0] seg2;				// Segments for DF, position, etc.
	 reg [3:0] HEX1;				// A digit for 7 seg display for DF, Position, etc.
	 reg [3:0] HEX2;				// A digit for 7 seg display for DF, Position, etc.
	 wire [3:0] sign;				// Divider for 7SD
	 reg [18:0] cnt;
	 
	 // ===========================================================================
	 // 										Implementation
	 // ===========================================================================	 
	 assign sign = (~SpeedIn[12]) ? 4'b1110 : 4'b1111;
	 
	 //  Displaying the BCD value
	 always @(posedge ck)
	 begin : Display_Process
			cnt <= cnt + 1'b1;
	 end
	 
	 
	 always @(cnt[18] or cnt[17] or vecDfIn or swDir)
	 case (cnt[18:17])
			2'b00 : HEX1 = vecDfIn[3:0];				// For duty factor
			2'b01 : HEX1 = vecDfIn[7:4];
			2'b10 : HEX1 = {3'b111, swDir};			// 1110 for positive, 1111 for negative
			default : HEX1 = 4'b1110;					// MSD off
	 endcase
	 

	 always @(cnt[18] or cnt[17] or SpeedIn or sign)
	 case (cnt[18:17])
			2'b00 : HEX2 = SpeedIn[3:0];				// For speed
			2'b01 : HEX2 = SpeedIn[7:4];
			2'b10 : HEX2 = SpeedIn[11:8];
			default : HEX2 = sign;						// Positive or negative
	 endcase
	 

	 always @(cnt[18] or cnt[17])
	 case (cnt[18:17])
			2'b00 : an = 4'b1110;
			2'b01 : an = 4'b1101;
			2'b10 : an = 4'b1011;
			default : an = 4'b0111;
	 endcase

	 // HEX-to-seven-segment decoder
	 // 
	 // segment encoding
	 //      0
	 //     ---  
	 //  5 |   | 1
	 //     ---   <- 6
	 //  4 |   | 2
	 //     ---
  	 //      3

	 //  ------------------------
	 always @(HEX1[3] or HEX1[2] or HEX1[1] or HEX1[0])
	 case (HEX1[3:0])
			4'b0000 : seg1 = 7'b1000000;		// 0
			4'b0001 : seg1 = 7'b1111001;		// 1
			4'b0010 : seg1 = 7'b0100100;		// 2
			4'b0011 : seg1 = 7'b0110000;		// 3
			4'b0100 : seg1 = 7'b0011001;		// 4
			4'b0101 : seg1 = 7'b0010010;		// 5
			4'b0110 : seg1 = 7'b0000010;		// 6
			4'b0111 : seg1 = 7'b1111000;		// 7
			4'b1000 : seg1 = 7'b0000000;		// 8
			4'b1001 : seg1 = 7'b0010000;		// 9
			4'b1111 : seg1 = 7'b0111111;		//  Minus sign
			default : seg1 = 7'b1111111;		//  Nothing for plus sign
	 endcase
	 
	 always @(HEX2[3] or HEX2[2] or HEX2[1] or HEX2[0])
	 case (HEX2[3:0])
			4'b0000 : seg2 = 7'b1000000;		// 0
			4'b0001 : seg2 = 7'b1111001;		// 1
			4'b0010 : seg2 = 7'b0100100;		// 2
			4'b0011 : seg2 = 7'b0110000;		// 3
			4'b0100 : seg2 = 7'b0011001;		// 4
			4'b0101 : seg2 = 7'b0010010;		// 5
			4'b0110 : seg2 = 7'b0000010;		// 6
			4'b0111 : seg2 = 7'b1111000;		// 7
			4'b1000 : seg2 = 7'b0000000;		// 8
			4'b1001 : seg2 = 7'b0010000;		// 9
			4'b1111 : seg2 = 7'b0111111;		//  Minus sign
			default : seg2 = 7'b1111111;		//  Nothing for plus sign
	 endcase

	 assign seg = (~swSelDisp) ? seg1 : seg2;

endmodule
